package info.edoardonosotti.android.tutorial.restfulws;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

// This background task will call remote methods and retrieve JSON responses
public class RPCTask extends AsyncTask<String, Void, String> {
	private Handler mHandler;
	
	public static String RESULT = "RESULT";
	private static String TAG = "RPCTask";
	
	public RPCTask(Handler handler) {
		mHandler = handler;
	}
	
	@Override
	protected String doInBackground(String... params) {
		HttpClient client = new DefaultHttpClient();
		
		URL url;
		try {
			url = new URL(params[0]);
		} catch (MalformedURLException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return null;
		}
		
		HttpHost host = new HttpHost(url.getHost(), url.getPort(), url.getProtocol());
        HttpRequest request = new HttpGet(url.toString());
        
        /*
         * TODO - Signing your request
         * This is where you should sign your request with OAuth or other signign protocols.
         * Should your Web Services be under HTTP Basic Auth, you can apply it on HttpClient
         * object, check AuthScope and BasicScheme documentation.
         */
        
        HttpResponse response;
		try {
			response = client.execute(host, request);
		} catch (ClientProtocolException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return null;
		}
        
        HttpEntity entity = response.getEntity();
        
        Object content;
        try {
			content = EntityUtils.toString(entity);
		} catch (ParseException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return null;
		} 
        
        return content.toString();
	}
	
	@Override
	protected void onPostExecute(String result) {
		super.onPostExecute(result);
		if (mHandler != null) {
			Bundle b = new Bundle();
			b.putString(RPCTask.RESULT, result);
			Message m = new Message();
			m.setData(b);
			mHandler.sendMessage(m);
		}
	}
}